@@ -145,7 +145,7 @@ conMLM.mlm <- function(object, constraints = NULL, se = "none",
                 b.unrestr   = b.unrestr,
                 b.restr     = b.unrestr,
                 residuals   = residuals, 
-                fitted      = object$fitted,
+                fitted      = object$fitted.values,
                 df.residual = df.residual,
                 R2.org      = R2.org, 
                 R2.reduced  = R2.org,
